# Generated by Django 3.2.23 on 2024-01-12 07:24

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0197_auto_20231130_2239'),
        ('integrated_channel', '0029_genericenterprisecustomerpluginconfiguration_show_course_price'),
    ]

    operations = [
        migrations.CreateModel(
            name='IntegratedChannelAPIRequestLogs',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('enterprise_customer_configuration_id', models.IntegerField(help_text='ID from the EnterpriseCustomerConfiguration model')),
                ('endpoint', models.TextField()),
                ('payload', models.TextField()),
                ('time_taken', models.DurationField()),
                ('api_record', models.OneToOneField(blank=True, help_text='Data pertaining to the transmissions API request response.', null=True, on_delete=django.db.models.deletion.CASCADE, to='integrated_channel.apiresponserecord')),
                ('enterprise_customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='enterprise.enterprisecustomer')),
            ],
        ),
    ]



"""
Views for Plotly auth.
"""

from time import time

import jwt
from edx_rbac.decorators import permission_required
from rest_framework import generics
from rest_framework.permissions import IsAuthenticated

from django.conf import settings
from django.http import JsonResponse

from enterprise.models import EnterpriseCustomer


class PlotlyAuthView(generics.GenericAPIView):
    """
    API to generate a signed token for an enterprise admin to use Plotly analytics.
    """
    permission_classes = (IsAuthenticated,)

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, enterprise_uuid: enterprise_uuid
    )
    def get(self, request, enterprise_uuid):
        """
        Generate auth token for plotly.
        """
        # This is a new secret key and will be only shared between LMS and our Plotly server.
        secret_key = settings.ENTERPRISE_PLOTLY_SECRET

        now = int(time())
        expires_in = 3600  # time in seconds after which token will be expired
        exp = now + expires_in

        CLAIMS = {
            "exp": exp,
            "iat": now
        }

        jwt_payload = dict({
            'enterprise_uuid': enterprise_uuid,
            'audit_data_reporting_enabled': self._is_audit_data_reporting_enabled(enterprise_uuid),
        }, **CLAIMS)

        token = jwt.encode(jwt_payload, secret_key, algorithm='HS512')
        json_payload = {'token': token}
        return JsonResponse(json_payload)

    @staticmethod
    def _is_audit_data_reporting_enabled(enterprise_uuid):
        """
        Check if audit data reporting is enabled for the enterprise.

        Args:
            enterprise_uuid (str): UUID of the enterprise.
        """
        enterprise = EnterpriseCustomer.objects.filter(uuid=enterprise_uuid).first()
        return getattr(enterprise, 'enable_audit_data_reporting', False)



"""
xAPI Client to send payload data.
"""

import logging

from tincan.remote_lrs import RemoteLRS

from integrated_channels.exceptions import ClientError

LOGGER = logging.getLogger(__name__)


class EnterpriseXAPIClient:
    """
    xAPI to send payload data and handle responses.
    """

    def __init__(self, lrs_configuration):
        """
        Initialize xAPI client.

        Arguments:
             lrs_configuration (XAPILRSConfiguration): Configuration object for xAPI LRS.
        """
        self.lrs_configuration = lrs_configuration

    @property
    def lrs(self):
        """
        LRS client instance to be used for sending statements.
        """
        return RemoteLRS(
            version=self.lrs_configuration.version,
            endpoint=self.lrs_configuration.endpoint,
            auth=self.lrs_configuration.authorization_header,
        )

    def save_statement(self, statement):
        """
        Save xAPI statement.

        Arguments:
            statement (EnterpriseStatement): xAPI Statement to send to the LRS.

        Raises:
            ClientError: If xAPI statement fails to save.
        """
        LOGGER.info(
            "[Integrated Channel][xAPI] Sending statement %s to LRS endpoint %s ",
            statement.to_json(),
            self.lrs.endpoint
        )
        response = self.lrs.save_statement(statement)

        if not response:
            raise ClientError('EnterpriseXAPIClient request failed.')

        return response



# Generated by Django 2.2.24 on 2021-09-09 15:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0009_auto_20210708_1639'),
    ]

    operations = [
        migrations.AddField(
            model_name='canvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255, null=True),
        ),
    ]



"""
Client for communicating with the Enterprise API.
"""

from collections import OrderedDict
from logging import getLogger
from urllib.parse import urljoin

from django.conf import settings
from django.core.cache import cache

from enterprise import utils
from enterprise.api_client.client import UserAPIClient

LOGGER = getLogger(__name__)


class EnterpriseApiClient(UserAPIClient):
    """
    The API client to make calls to the Enterprise API.
    """

    API_BASE_URL = urljoin(f"{settings.LMS_INTERNAL_ROOT_URL}/", "enterprise/api/v1/")
    APPEND_SLASH = True

    ENTERPRISE_CUSTOMER_ENDPOINT = 'enterprise-customer'
    ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT = 'enterprise_catalogs'

    DEFAULT_VALUE_SAFEGUARD = object()

    def get_content_metadata(self, enterprise_customer, enterprise_catalogs=None):
        """
        Return all content metadata contained in the catalogs associated with the EnterpriseCustomer.

        Arguments:
            enterprise_customer (EnterpriseCustomer): The EnterpriseCustomer to return content metadata for.
            enterprise_catalogs (EnterpriseCustomerCatalog): Optional list of EnterpriseCustomerCatalog objects.

        Returns:
            list: List of dicts containing content metadata.
        """
        content_metadata = OrderedDict()
        enterprise_customer_catalogs = enterprise_catalogs or enterprise_customer.enterprise_customer_catalogs.all()
        for enterprise_customer_catalog in enterprise_customer_catalogs:
            response = self._load_data(
                self.ENTERPRISE_CUSTOMER_CATALOGS_ENDPOINT,
                resource_id=str(enterprise_customer_catalog.uuid),
                traverse_pagination=True,
                querystring={'page_size': 1000},
            )

            for item in response['results']:
                content_id = utils.get_content_metadata_item_id(item)
                content_metadata[content_id] = item

        return list(content_metadata.values())

    @UserAPIClient.refresh_token
    def _load_data(
            self,
            resource,
            detail_resource=None,
            resource_id=None,
            querystring=None,
            traverse_pagination=False,
            default=DEFAULT_VALUE_SAFEGUARD,
    ):
        """
        Loads a response from a call to one of the Enterprise endpoints.

        :param resource: The endpoint resource name.
        :param detail_resource: The sub-resource to append to the path.
        :param resource_id: The resource ID for the specific detail to get from the endpoint.
        :param querystring: Optional query string parameters.
        :param traverse_pagination: Whether to traverse pagination or return paginated response.
        :param default: The default value to return in case of no response content.
        :return: Data returned by the API.
        """
        default_val = default if default != self.DEFAULT_VALUE_SAFEGUARD else {}
        querystring = querystring if querystring else {}

        cache_key = utils.get_cache_key(
            resource=resource,
            querystring=querystring,
            traverse_pagination=traverse_pagination,
            resource_id=resource_id
        )
        response = cache.get(cache_key)
        if not response:
            # Response is not cached, so make a call.
            api_url = self.get_api_url(f"{resource}/{resource_id}/{detail_resource if detail_resource else ''}")
            response = self.client.get(api_url, params=querystring)
            response.raise_for_status()
            response = response.json()
            if traverse_pagination:
                results = utils.traverse_pagination(response, self.client, api_url)
                response = {
                    'count': len(results),
                    'next': 'None',
                    'previous': 'None',
                    'results': results,
                }
            if response:
                # Now that we've got a response, cache it.
                cache.set(cache_key, response, settings.ENTERPRISE_API_CACHE_TIMEOUT)
        return response or default_val



"""
Enterprise Integrated Channel SAP SuccessFactors Django application initialization.
"""

from django.apps import AppConfig


class SAPSuccessFactorsConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel SAP SuccessFactors Django application.
    """
    name = 'integrated_channels.sap_success_factors'
    verbose_name = "Enterprise SAP SuccessFactors Integration"



# Generated by Django 3.2.15 on 2022-10-31 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed', '0025_auto_20221031_1855'),
    ]

    operations = [
        migrations.RenameField(
            model_name='degreedlearnerdatatransmissionaudit',
            old_name='completed_timestamp',
            new_name='degreed_completed_timestamp',
        ),
        migrations.AddField(
            model_name='degreedlearnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



"""
Views for the Enterprise Subsidy Fulfillment API.
"""

from edx_rbac.decorators import permission_required
from edx_rbac.mixins import PermissionRequiredMixin
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.exceptions import ValidationError
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_204_NO_CONTENT, HTTP_404_NOT_FOUND, HTTP_500_INTERNAL_SERVER_ERROR

from django.http import Http404
from django.shortcuts import get_object_or_404
from django.utils.dateparse import parse_datetime
from django.utils.functional import cached_property
from django.utils.translation import gettext as _

from enterprise import models
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseWrapperApiViewSet
from enterprise.logging import getEnterpriseLogger
from enterprise.utils import NotConnectedToOpenEdX, get_request_value
from enterprise_learner_portal.utils import CourseRunProgressStatuses, get_course_run_status

try:
    from common.djangoapps.course_modes.models import CourseMode
    from common.djangoapps.student.models import CourseEnrollment
    from lms.djangoapps.certificates.api import get_certificate_for_user
    from openedx.core.djangoapps.content.course_overviews.api import get_course_overviews
    from openedx.core.djangoapps.enrollments import api as enrollment_api
except ImportError:
    get_course_overviews = None
    get_certificate_for_user = None
    CourseEnrollment = None
    CourseMode = None
    enrollment_api = None

LOGGER = getEnterpriseLogger(__name__)


class EnrollmentModificationException(Exception):
    """
    An exception that represents an error when modifying the state
    of an enrollment via the EnrollmentApiClient.
    """


class EnterpriseSubsidyFulfillmentViewSet(PermissionRequiredMixin, EnterpriseWrapperApiViewSet):
    """
    General API views for subsidized enterprise course enrollments.

    Supported operations:
        * Fetch a subsidy fulfillment record by uuid.
            /enterprise/api/v1/enterprise-subsidy-fulfillment/{fulfillment_source_uuid}/
        * Cancel a subsidy fulfillment enrollment record by uuid.
            /enterprise/api/v1/enterprise-subsidy-fulfillment/{fulfillment_source_uuid}/cancel-fulfillment/
        * Fetch all unenrolled subsidy fulfillment records.
            /enterprise/api/v1/operator/enterprise-subsidy-fulfillment/unenrolled/

    Cancel and fetch endpoints require a fulfillment source uuid query parameter. Fetching unenrollments supports
    an optional ``unenrolled_after`` query parameter to filter the returned queryset down to only enterprise
    enrollments unenrolled after the supplied datetime.

    Arguments (Fetch & Cancel):
        fulfillment_source_uuid (str): The uuid of the subsidy fulfillment record.

    Arguments (Unenrolled):
        unenrolled_after (str): A datetime string. Only return enrollments unenrolled after this time.

    Returns (Fetch):
        (Response): JSON response containing the subsidy fulfillment record.

    Returns (Unenrolled):
        (Response): JSON list response containing the unenrolled subsidy fulfillment records.

        .. code-block::

            api_response = [
                {
                    enterprise_course_enrollment: {
                        enterprise_customer_user: <user_id>,
                        course_id: <course_id>,
                        unenrolled: <datetime>
                        created: <datetime>
                    }
                    license_uuid/transaction_id: <uuid>,
                    uuid: <uuid>,
                },
            ]

    Raises
        (Http404): If the subsidy fulfillment record does not exist or if subsidy fulfillment exists under a separate
        enterprise.
        (Http403): If the requesting user does not have the appropriate permissions.
        (EnrollmentModificationException): If something goes wrong while updating the platform CourseEnrollment object.
    """
    def get_permission_required(self):
        """
        Return specific permission name based on the view being requested
        """
        if self.action == 'unenrolled':
            return ['enterprise.can_manage_enterprise_fulfillments']
        elif self.action == 'retrieve':
            return ['enterprise.can_access_admin_dashboard']
        elif self.action == 'cancel_enrollment':
            return ['enterprise.can_enroll_learners']
        return []

    def get_permission_object(self):
        """
        Depending on the requested view, returns a record identifier to check perms against.
        """
        if self.request.user.is_staff:
            return None

        if self.action in ('retrieve', 'cancel_enrollment'):
            enrollment_record = self.requested_fulfillment_source.enterprise_course_enrollment
            return str(enrollment_record.enterprise_customer_user.enterprise_customer.uuid)
        else:
            return None

    @cached_property
    def requested_fulfillment_source(self):
        """
        Computes and caches the requested fulfillment source record for this request.
        """
        return self._get_single_fulfillment_by_uuid()

    def _get_single_fulfillment_by_uuid(self):
        """
        Returns a single LearnerCreditCourseEnrollment or LicensedCourseEnrollment associated
        with the requested ``fulfillment_source_uuid``.

        Raises: a 404 if no such record can be found.
        """
        fulfillment_source_uuid = self.kwargs.get('fulfillment_source_uuid')

        # Get learner credit fulfillments under the supplied fulfillment source uuid.
        learner_credit_enrollment = models.LearnerCreditEnterpriseCourseEnrollment.objects.filter(
            uuid=fulfillment_source_uuid,
        ).first()
        if learner_credit_enrollment:
            return learner_credit_enrollment

        # If no LC fulfillment records, look for licensed enrollment records.
        licensed_enrollment = models.LicensedEnterpriseCourseEnrollment.objects.filter(
            uuid=fulfillment_source_uuid,
        ).first()
        if licensed_enrollment:
            return licensed_enrollment

        raise Http404('No fulfillment record matches the provided UUID.')

    def get_subsidy_fulfillment_serializer_class(self):
        """
        Fetch the correct serializer class based on the subsidy type.
        """
        fulfillment_source_uuid = self.kwargs.get('fulfillment_source_uuid')

        learner_credit_enrollments = models.LearnerCreditEnterpriseCourseEnrollment.objects.filter(
            uuid=fulfillment_source_uuid
        )
        if len(learner_credit_enrollments):
            return serializers.LearnerCreditEnterpriseCourseEnrollmentReadOnlySerializer
        licensed_enrollments = models.LicensedEnterpriseCourseEnrollment.objects.filter(
            uuid=fulfillment_source_uuid
        )
        if len(licensed_enrollments):
            return serializers.LicensedEnterpriseCourseEnrollmentReadOnlySerializer

        raise ValidationError('No enrollment found for the given fulfillment source uuid.', code=HTTP_404_NOT_FOUND)

    def _get_unenrolled_fulfillments(self):
        """
        Return the unenrolled subsidy fulfillment records.

        Optionally reads the "unenrolled_after" query param to return records unenrolled after a specified date.

        Furthermore, filter out fulfillments with active related student.CourseEnrollment records.

        Returns:
          Generator of EnterpriseFulfillmentSource records representing only unenrolled fulfillments.
        """
        # Adding licensed enrollment support for future implementations
        if self.request.query_params.get('retrieve_licensed_enrollments'):
            enrollment_table = models.LicensedEnterpriseCourseEnrollment
        else:
            enrollment_table = models.LearnerCreditEnterpriseCourseEnrollment

        unenrolled_queryset = None
        # Apply a modified filter if one is provided via query params
        if self.request.query_params.get('unenrolled_after'):
            unenrolled_queryset = enrollment_table.objects.filter(
                enterprise_course_enrollment__unenrolled_at__gte=self.request.query_params.get('unenrolled_after')
            )
        else:
            unenrolled_queryset = enrollment_table.objects.filter(
                enterprise_course_enrollment__unenrolled_at__isnull=False,
            )
        # Make sure the related enrollment (if it exists) is not active, and exclude it from the results if so.
        #
        # Note: There's no FK between an enterprise enrollment and course enrollment, and the only way to join them is
        # on two distinct join keys (user_id & course_id). There's no native ORM way to do this join, short of using a
        # raw() SQL query. Therefore, we will just make O(N) SQL queries where N = number of recently unenrolled
        # fulfillments.
        unenrolled_fulfillments = (
            fulfillment for fulfillment in unenrolled_queryset
            if not (
                fulfillment.enterprise_course_enrollment
                and fulfillment.enterprise_course_enrollment.course_enrollment
                and fulfillment.enterprise_course_enrollment.course_enrollment.is_active
            )
        )
        return unenrolled_fulfillments

    def get_unenrolled_fulfillment_serializer_class(self):
        """
        Fetch the correct recently unenrolled serializer class based on provided querysets.
        """
        if self.request.query_params.get('retrieve_licensed_enrollments'):
            return serializers.LicensedEnterpriseCourseEnrollmentReadOnlySerializer
        else:
            return serializers.LearnerCreditEnterpriseCourseEnrollmentReadOnlySerializer

    @action(methods=['GET'], detail=False)
    def unenrolled(self, request, *args, **kwargs):
        """
        List all unenrolled subsidy fulfillments.
            /enterprise/api/v1/operator/enterprise-subsidy-fulfillment/unenrolled/

        Args:
            modified (str): A datetime string. Only return enrollments modified after this time.
            retrieve_licensed_enrollments (bool): If true, return data related to licensed enrollments instead of
                learner credit
        """
        queryset = self._get_unenrolled_fulfillments()
        serializer_class = self.get_unenrolled_fulfillment_serializer_class()
        serializer = serializer_class(queryset, many=True)
        return Response(serializer.data)

    def retrieve(self, request, fulfillment_source_uuid, *args, **kwargs):  # pylint: disable=unused-argument
        """
        Retrieve a single subsidized enrollment.
            /enterprise/api/v1/subsidy-fulfillment/{fulfillment_source_uuid}/
        """
        try:
            serializer_class = self.get_subsidy_fulfillment_serializer_class()
            serialized_object = serializer_class(self.requested_fulfillment_source)
        except ValidationError as exc:
            return Response(
                status=HTTP_404_NOT_FOUND,
                data={'detail': exc.detail}
            )
        return Response(serialized_object.data)

    @action(methods=['post'], detail=True)
    def cancel_enrollment(self, request, fulfillment_source_uuid):  # pylint: disable=unused-argument
        """
        Cancel a single subsidized enrollment. Assumes fulfillment source has a valid enterprise enrollment.
        /enterprise/api/v1/enterprise-subsidy-fulfillment/{fulfillment_source_uuid}/cancel-fulfillment/
        """
        try:
            subsidy_fulfillment = self.requested_fulfillment_source
            if subsidy_fulfillment.is_revoked:
                return Response(
                    status=HTTP_200_OK,
                    data={'detail': 'Enrollment is already canceled.'}
                )
        except ValidationError as exc:
            return Response(
                status=HTTP_404_NOT_FOUND,
                data={'detail': exc.detail}
            )

        try:
            username = subsidy_fulfillment.enterprise_course_enrollment.enterprise_customer_user.username
            enrollment_api.update_enrollment(
                username,
                subsidy_fulfillment.enterprise_course_enrollment.course_id,
                is_active=False,
            )
            subsidy_fulfillment.revoke()
        except Exception as exc:  # pylint: disable=broad-except
            msg = (
                f'Subsidized enrollment terminations error: unable to unenroll User {username} '
                f'from Course {subsidy_fulfillment.enterprise_course_enrollment.course_id} because: {str(exc)}'
            )
            LOGGER.error(msg)
            return Response(msg, status=HTTP_500_INTERNAL_SERVER_ERROR)
        return Response(status=HTTP_204_NO_CONTENT)


class LicensedEnterpriseCourseEnrollmentViewSet(EnterpriseWrapperApiViewSet):
    """
    API views for the ``licensed-enterprise-course-enrollment`` API endpoint.
    """

    queryset = models.LicensedEnterpriseCourseEnrollment.objects.all()
    serializer_class = serializers.LicensedEnterpriseCourseEnrollmentReadOnlySerializer
    REQ_EXP_LICENSE_UUIDS_PARAM = 'expired_license_uuids'
    OPT_IGNORE_ENROLLMENTS_MODIFIED_AFTER_PARAM = 'ignore_enrollments_modified_after'

    class EnrollmentTerminationStatus:
        """
        Defines statuses related to enrollment states during the course unenrollment process.
        """
        COURSE_COMPLETED = 'course already completed'
        MOVED_TO_AUDIT = 'moved to audit'
        UNENROLLED = 'unenrolled'
        UNENROLL_FAILED = 'unenroll_user_from_course returned false.'

    @staticmethod
    def _validate_license_revoke_data(request_data):
        """
        Ensures the request data contains the necessary information.

        Arguments:
            request_data (dict): A dictionary of data passed to the request
        """
        user_id = request_data.get('user_id')
        enterprise_id = request_data.get('enterprise_id')

        if not user_id or not enterprise_id:
            msg = 'user_id and enterprise_id must be provided.'
            return Response(msg, status=status.HTTP_400_BAD_REQUEST)

        return None

    @staticmethod
    def _has_user_completed_course_run(enterprise_enrollment, course_overview):
        """
        Returns True if the user who is enrolled in the given course has already
        completed this course, false otherwise.  The course may be "completed"
        if the user earned a certificate, or if the course run has ended.

        Args:
            enterprise_enrollment (EnterpriseCourseEnrollment): The enrollment object for which we check
                if the associated user has completed the given course.
            course_overview (CourseOverview): The course overview of which we are checking completion.  We need this
                to check certificate status.  It's a model defined in edx-platform.
        """
        certificate_info = get_certificate_for_user(
            enterprise_enrollment.enterprise_customer_user.username,
            course_overview.get('id'),
        ) or {}
        course_run_status = get_course_run_status(
            course_overview,
            certificate_info,
            enterprise_enrollment,
        )

        return course_run_status == CourseRunProgressStatuses.COMPLETED

    def _enrollments_by_course_for_licensed_user(self, enterprise_customer_user):
        """
        Helper method to return a dictionary mapping course ids to EnterpriseCourseEnrollments
        for each licensed enrollment associated with the given enterprise user.

        Args:
            enterprise_customer_user (EnterpriseCustomerUser): The user for which we are fetching enrollments.
        """
        licensed_enrollments = models.LicensedEnterpriseCourseEnrollment.enrollments_for_user(
            enterprise_customer_user
        )
        return {
            enrollment.enterprise_course_enrollment.course_id: enrollment.enterprise_course_enrollment
            for enrollment in licensed_enrollments
        }

    def _terminate_enrollment(self, enterprise_enrollment, course_overview):
        """
        Helper method that switches the given enrollment to audit track, or, if
        no audit track exists for the given course, deletes the enrollment.
        Will do nothing if the user has already "completed" the course run.

        Args:
            enterprise_enrollment (EnterpriseCourseEnrollment): The enterprise enrollment which we attempt to revoke.
            course_overview (CourseOverview): The course overview object associated with the enrollment. Used
                to check for course completion.
        """
        course_run_id = course_overview.get('id')
        enterprise_customer_user = enterprise_enrollment.enterprise_customer_user
        audit_mode = CourseMode.AUDIT
        enterprise_id = enterprise_customer_user.enterprise_customer.uuid

        log_message_kwargs = {
            'user': enterprise_customer_user.username,
            'enterprise': enterprise_id,
            'course_id': course_run_id,
            'mode': audit_mode,
        }

        if self._has_user_completed_course_run(enterprise_enrollment, course_overview):
            LOGGER.info(
                'enrollment termination: not updating enrollment in {course_id} for User {user} '
                'in Enterprise {enterprise}, course is already complete.'.format(**log_message_kwargs)
            )
            return self.EnrollmentTerminationStatus.COURSE_COMPLETED

        if CourseMode.mode_for_course(course_run_id, audit_mode):
            try:
                enrollment_api.update_enrollment(
                    username=enterprise_customer_user.username,
                    course_id=course_run_id,
                    mode=audit_mode,
                )
                LOGGER.info(
                    'Enrollment termination: updated LMS enrollment for User {user} and Enterprise {enterprise} '
                    'in Course {course_id} to Course Mode {mode}.'.format(**log_message_kwargs)
                )
                return self.EnrollmentTerminationStatus.MOVED_TO_AUDIT
            except Exception as exc:
                msg = (
                    'Enrollment termination: unable to update LMS enrollment for User {user} and '
                    'Enterprise {enterprise} in Course {course_id} to Course Mode {mode} because: {reason}'.format(
                        reason=str(exc),
                        **log_message_kwargs
                    )
                )
                LOGGER.error('{msg}: {exc}'.format(msg=msg, exc=exc))
                raise EnrollmentModificationException(msg) from exc
        else:
            try:
                enrollment_api.update_enrollment(
                    username=enterprise_customer_user.username,
                    course_id=course_run_id,
                    is_active=False
                )
                LOGGER.info(
                    'Enrollment termination: successfully unenrolled User {user}, in Enterprise {enterprise} '
                    'from Course {course_id} that contains no audit mode.'.format(**log_message_kwargs)
                )
                return self.EnrollmentTerminationStatus.UNENROLLED
            except Exception as exc:
                msg = (
                    'Enrollment termination: unable to unenroll User {user} in Enterprise {enterprise} '
                    'from Course {course_id}  because: {reason}'.format(
                        reason=str(exc),
                        **log_message_kwargs
                    )
                )
                LOGGER.error('{msg}: {exc}'.format(msg=msg, exc=exc))
                raise EnrollmentModificationException(msg) from exc

    def _course_enrollment_modified_at_by_user_and_course_id(self, licensed_enrollments):
        """
        Returns a dict containing the last time a course enrollment was modified.
        The keys are in the form of f'{user_id}{course_id}'.
        """
        enterprise_course_enrollments = [
            licensed_enrollment.enterprise_course_enrollment for licensed_enrollment in licensed_enrollments
        ]
        user_ids = [str(ece.enterprise_customer_user.user_id) for ece in enterprise_course_enrollments]
        course_ids = [str(ece.course_id) for ece in enterprise_course_enrollments]
        course_enrollment_histories = CourseEnrollment.history.filter(
            user_id__in=user_ids,
            course_id__in=course_ids
        ).order_by('-history_date')

        result = {}

        for history in course_enrollment_histories:
            user_id = history.user_id
            course_id = str(history.course_id)
            key = f'{user_id}{course_id}'
            if key not in result:
                result[key] = history.history_date

        return result

    @action(methods=['post'], detail=False)
    @permission_required('enterprise.can_access_admin_dashboard', fn=lambda request: request.data.get('enterprise_id'))
    def license_revoke(self, request, *args, **kwargs):
        """
        Changes the mode for a user's licensed enterprise course enrollments to the "audit" course mode,
        or unenroll the user if no audit mode exists for a given course.

        Will return a response with status 200 if no errors were encountered while modifying the course enrollment,
        or a 422 if any errors were encountered.  The content of the response is of the form::

            {
                'course-v1:puppies': {'success': true, 'message': 'unenrolled'},
                'course-v1:birds': {'success': true, 'message': 'moved to audit'},
                'course-v1:kittens': {'success': true, 'message': 'course already completed'},
                'course-v1:snakes': {'success': false, 'message': 'unenroll_user_from_course returned false'},
                'course-v1:lizards': {'success': false, 'message': 'Some other exception'},
            }

        The first four messages are the values of constants that a client may expect to receive and parse accordingly.
        """
        dependencies = [
            CourseMode, get_certificate_for_user, get_course_overviews, enrollment_api
        ]
        if not all(dependencies):
            raise NotConnectedToOpenEdX(
                _('To use this endpoint, this package must be '
                  'installed in an Open edX environment.')
            )

        request_data = request.data.copy()
        invalid_response = self._validate_license_revoke_data(request_data)
        if invalid_response:
            return invalid_response

        user_id = request_data.get('user_id')
        enterprise_id = request_data.get('enterprise_id')

        enterprise_customer_user = get_object_or_404(
            models.EnterpriseCustomerUser,
            user_id=user_id,
            enterprise_customer=enterprise_id,
        )
        enrollments_by_course_id = self._enrollments_by_course_for_licensed_user(enterprise_customer_user)

        revocation_results = {}
        any_failures = False
        for course_overview in get_course_overviews(list(enrollments_by_course_id.keys())):
            course_id = str(course_overview.get('id'))
            enterprise_enrollment = enrollments_by_course_id.get(course_id)
            try:
                revocation_status = self._terminate_enrollment(enterprise_enrollment, course_overview)
                revocation_results[course_id] = {'success': True, 'message': revocation_status}
                if revocation_status != self.EnrollmentTerminationStatus.COURSE_COMPLETED:
                    enterprise_enrollment.license.revoke()
            except EnrollmentModificationException as exc:
                revocation_results[course_id] = {'success': False, 'message': str(exc)}
                any_failures = True

        status_code = status.HTTP_200_OK if not any_failures else status.HTTP_422_UNPROCESSABLE_ENTITY
        return Response(revocation_results, status=status_code)

    @action(methods=['post'], detail=False)
    @permission_required('enterprise.can_enroll_learners')
    def bulk_licensed_enrollments_expiration(self, request):
        """
        Changes the mode for licensed enterprise course enrollments to the "audit" course mode,
        or unenroll the user if no audit mode exists for each expired license uuid

        Args:
            expired_license_uuids: The expired license uuids.
            ignore_enrollments_modified_after: All course enrollments modified past this given date will be ignored,
                                               i.e. the enterprise subscription plan expiration date.
        """

        dependencies = [
            CourseEnrollment, CourseMode, get_certificate_for_user, get_course_overviews, enrollment_api
        ]
        if not all(dependencies):
            raise NotConnectedToOpenEdX(
                _('To use this endpoint, this package must be '
                  'installed in an Open edX environment.')
            )

        expired_license_uuids = get_request_value(request, self.REQ_EXP_LICENSE_UUIDS_PARAM, '')
        ignore_enrollments_modified_after = get_request_value(
            request,
            self.OPT_IGNORE_ENROLLMENTS_MODIFIED_AFTER_PARAM,
            None
        )

        if not expired_license_uuids:
            return Response(
                'Parameter {} must be provided'.format(self.REQ_EXP_LICENSE_UUIDS_PARAM),
                status=status.HTTP_400_BAD_REQUEST
            )

        if ignore_enrollments_modified_after:
            ignore_enrollments_modified_after = parse_datetime(ignore_enrollments_modified_after)
            if not ignore_enrollments_modified_after:
                return Response(
                    'Parameter {} is malformed, please provide a date in ISO-8601 format'.format(
                        self.OPT_IGNORE_ENROLLMENTS_MODIFIED_AFTER_PARAM
                    ),
                    status=status.HTTP_400_BAD_REQUEST
                )

        licensed_enrollments = models.LicensedEnterpriseCourseEnrollment.objects.filter(
            license_uuid__in=expired_license_uuids
        ).select_related('enterprise_course_enrollment')

        course_overviews = get_course_overviews(
            list(licensed_enrollments.values_list('enterprise_course_enrollment__course_id', flat=True))
        )
        indexed_overviews = {overview.get('id'): overview for overview in course_overviews}

        course_enrollment_modified_at_by_user_and_course_id = \
            self._course_enrollment_modified_at_by_user_and_course_id(
                licensed_enrollments
            ) if ignore_enrollments_modified_after else {}

        any_failures = False

        for licensed_enrollment in licensed_enrollments:
            enterprise_course_enrollment = licensed_enrollment.enterprise_course_enrollment
            user_id = enterprise_course_enrollment.enterprise_customer_user.user_id
            course_id = enterprise_course_enrollment.course_id
            course_overview = indexed_overviews.get(course_id)

            if licensed_enrollment.is_revoked:
                LOGGER.info(
                    'Enrollment termination: not updating enrollment in {} for User {} '
                    'licensed enterprise enrollment has already been revoked in the past.'.format(
                        course_id,
                        user_id
                    )
                )
                continue

            if ignore_enrollments_modified_after:
                key = f'{user_id}{course_id}'
                course_enrollment_modified_at = course_enrollment_modified_at_by_user_and_course_id[key]
                if course_enrollment_modified_at >= ignore_enrollments_modified_after:
                    LOGGER.info(
                        'Enrollment termination: not updating enrollment in {} for User {} '
                        'course enrollment has been modified past {}.'.format(
                            course_id,
                            user_id,
                            ignore_enrollments_modified_after
                        )
                    )
                    continue

            try:
                termination_status = self._terminate_enrollment(enterprise_course_enrollment, course_overview)
                license_uuid = enterprise_course_enrollment.license.license_uuid
                LOGGER.info(
                    f"EnterpriseCourseEnrollment record with enterprise license {license_uuid} "
                    f"unenrolled to status {termination_status}."
                )
                if termination_status != self.EnrollmentTerminationStatus.COURSE_COMPLETED:
                    enterprise_course_enrollment.license.revoke()
            except EnrollmentModificationException as exc:
                LOGGER.error(
                    f"Failed to unenroll EnterpriseCourseEnrollment record for enterprise license "
                    f"{enterprise_course_enrollment.license.license_uuid}. error message {str(exc)}."
                )
                any_failures = True

        status_code = status.HTTP_200_OK if not any_failures else status.HTTP_422_UNPROCESSABLE_ENTITY
        return Response(status=status_code)



"""
Python API for doing CRUD operations on roles and user role assignments.
"""
from cache_memoize import cache_memoize

from enterprise.constants import (
    ENTERPRISE_ADMIN_ROLE,
    ENTERPRISE_LEARNER_ROLE,
    ENTERPRISE_OPERATOR_ROLE,
    SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE,
    SYSTEM_ENTERPRISE_PROVISIONING_ADMIN_ROLE,
)
from enterprise.models import SystemWideEnterpriseRole, SystemWideEnterpriseUserRoleAssignment


# django-cache-memoize lets us explicitly declare a prefix
# to use as part of the cache key (rather than using the function name),
# which we're doing here just to be extra-careful about cache key collisions.
# https://github.com/peterbe/django-cache-memoize/blob/79c19f0314ff09d3f67d995f8d21f213e4eec3c8/src/cache_memoize/__init__.py#L99-L109
@cache_memoize(60 * 60, prefix='enterprise_system_wide_role')
def get_or_create_system_wide_role(role_name):
    """
    Gets or creates the `SystemWideEnterpriseRole` with the given name.
    Caches the result for 60 minutes in the Django cache.
    Returns the role object.
    """
    return SystemWideEnterpriseRole.objects.get_or_create(name=role_name)[0]


def admin_role():
    """ Return the enterprise admin role. """
    return get_or_create_system_wide_role(ENTERPRISE_ADMIN_ROLE)


def learner_role():
    """ Returns the enterprise learner role. """
    return get_or_create_system_wide_role(ENTERPRISE_LEARNER_ROLE)


def openedx_operator_role():
    """ Returns the enterprise openedx operator role. """
    return get_or_create_system_wide_role(ENTERPRISE_OPERATOR_ROLE)


def catalog_admin_role():
    """ Returns the enterprise catalog admin role. """
    return get_or_create_system_wide_role(SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE)


def provisioning_admin_role():
    """ Returns the provisioning admin role. """
    return get_or_create_system_wide_role(SYSTEM_ENTERPRISE_PROVISIONING_ADMIN_ROLE)


def roles_by_name():
    """
    Returns a mapping of system wide roles by name.
    """
    return {
        ENTERPRISE_ADMIN_ROLE: admin_role(),
        ENTERPRISE_LEARNER_ROLE: learner_role(),
        ENTERPRISE_OPERATOR_ROLE: openedx_operator_role(),
        SYSTEM_ENTERPRISE_CATALOG_ADMIN_ROLE: catalog_admin_role(),
        SYSTEM_ENTERPRISE_PROVISIONING_ADMIN_ROLE: provisioning_admin_role(),
    }


def assign_learner_role(user, enterprise_customer=None, applies_to_all_contexts=False):
    """
    Assigns the given user the `enterprise_learner` role in the given customer.
    """
    return SystemWideEnterpriseUserRoleAssignment.objects.get_or_create(
        user=user,
        role=learner_role(),
        enterprise_customer=enterprise_customer,
        applies_to_all_contexts=applies_to_all_contexts,
    )


def assign_admin_role(user, enterprise_customer=None, applies_to_all_contexts=False):
    """
    Assigns the given user the `enterprise_admin` role in the given customer.
    """
    return SystemWideEnterpriseUserRoleAssignment.objects.get_or_create(
        user=user,
        role=admin_role(),
        enterprise_customer=enterprise_customer,
        applies_to_all_contexts=applies_to_all_contexts,
    )


def delete_role_assignment(user, role, enterprise_customer=None):
    """
    Deletes the given role assignment for the given user in the given enterprise customer.
    If `enterprise_customer` is null, will delete *every* role assignment for this user.
    """
    kwargs = {
        'user': user,
        'role': role,
    }
    if enterprise_customer:
        kwargs['enterprise_customer'] = enterprise_customer

    try:
        SystemWideEnterpriseUserRoleAssignment.objects.filter(**kwargs).delete()
    except SystemWideEnterpriseUserRoleAssignment.DoesNotExist:
        pass


def delete_learner_role_assignment(user, enterprise_customer=None):
    """
    Deletes the learner role assignment for the given user in the given enterprise customer.
    If `enterprise_customer` is null, will delete *every* learner role assignment for this user.
    """
    delete_role_assignment(user, learner_role(), enterprise_customer)


def delete_admin_role_assignment(user, enterprise_customer=None):
    """
    Deletes the admin role assignment for the given user in the given enterprise customer.
    If `enterprise_customer` is null, will delete *every* admin role assignment for this user.
    """
    delete_role_assignment(user, admin_role(), enterprise_customer)



# Generated by Django 2.2.14 on 2020-08-03 17:24

from django.db import migrations
from django.db.models import Q


def copy_primary_secondary_colors(apps, schema_editor):
    EnterpriseCustomerBrandingConfiguration = apps.get_model('enterprise', 'EnterpriseCustomerBrandingConfiguration')
    for branding_config in EnterpriseCustomerBrandingConfiguration.objects.filter(
        Q(banner_background_color__isnull=False) | Q(banner_border_color__isnull=False)
    ):
        branding_config.primary_color = branding_config.banner_background_color
        branding_config.secondary_color = branding_config.banner_border_color
        branding_config.save()


def clear_primary_secondary_colors(apps, schema_editor):
    EnterpriseCustomerBrandingConfiguration = apps.get_model('enterprise', 'EnterpriseCustomerBrandingConfiguration')
    for branding_config in EnterpriseCustomerBrandingConfiguration.objects.filter(
        Q(primary_color__isnull=False) | Q(secondary_color__isnull=False)
    ):
        branding_config.primary_color = None
        branding_config.secondary_color = None
        branding_config.save()



class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0105_add_branding_config_color_fields'),
    ]

    operations = [
        migrations.RunPython(copy_primary_secondary_colors, clear_primary_secondary_colors)
    ]



# Generated by Django 3.2.22 on 2024-01-25 09:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0030_integratedchannelapirequestlogs'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='integratedchannelapirequestlogs',
            options={'verbose_name_plural': 'Integrated channels API request logs'},
        ),
        migrations.RemoveField(
            model_name='integratedchannelapirequestlogs',
            name='api_record',
        ),
        migrations.AddField(
            model_name='integratedchannelapirequestlogs',
            name='response_body',
            field=models.TextField(blank=True, help_text='API call response body', null=True),
        ),
        migrations.AddField(
            model_name='integratedchannelapirequestlogs',
            name='status_code',
            field=models.PositiveIntegerField(blank=True, help_text='API call response HTTP status code', null=True),
        ),
        migrations.AlterField(
            model_name='integratedchannelapirequestlogs',
            name='time_taken',
            field=models.FloatField(),
        ),
    ]



# Generated by Django 3.2.23 on 2024-03-29 15:37

from django.db import migrations


def populate_decrypted_fields(apps, schema_editor):
    """
    Populates the encryption fields with the data previously stored in database.
    """
    Degreed2EnterpriseCustomerConfiguration = apps.get_model('degreed2', 'Degreed2EnterpriseCustomerConfiguration')

    for degreed2_enterprise_configuration in Degreed2EnterpriseCustomerConfiguration.objects.all():
        degreed2_enterprise_configuration.decrypted_client_id = degreed2_enterprise_configuration.client_id
        degreed2_enterprise_configuration.decrypted_client_secret = degreed2_enterprise_configuration.client_secret
        degreed2_enterprise_configuration.save()


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0026_auto_20240329_1537'),
    ]

    operations = [
        migrations.RunPython(populate_decrypted_fields, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 3.2.20 on 2023-09-21 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0185_auto_20230921_1007'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerssoconfiguration',
            name='entity_id',
            field=models.CharField(blank=True, help_text='The entity id of the identity provider.', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='enterprisecustomerssoconfiguration',
            name='metadata_url',
            field=models.CharField(blank=True, help_text='The metadata url of the identity provider.', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='entity_id',
            field=models.CharField(blank=True, help_text='The entity id of the identity provider.', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='metadata_url',
            field=models.CharField(blank=True, help_text='The metadata url of the identity provider.', max_length=255, null=True),
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-14 20:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0183_auto_20230906_1435'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomerssoconfiguration',
            name='first_name_attribute',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='first_name_attribute',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
    ]



"""
Views for coupon codes.
"""

from smtplib import SMTPException

from edx_rbac.decorators import permission_required
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import permissions
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_400_BAD_REQUEST, HTTP_500_INTERNAL_SERVER_ERROR
from rest_framework.views import APIView

from django.apps import apps
from django.core import mail
from django.utils.translation import gettext as _

from enterprise.api.throttles import ServiceUserThrottle
from enterprise.api.utils import create_message_body
from enterprise.errors import CodesAPIRequestError
from enterprise.logging import getEnterpriseLogger
from enterprise.utils import get_request_value

LOGGER = getEnterpriseLogger(__name__)


class CouponCodesView(APIView):
    """
    API to request coupon codes.
    """
    permission_classes = (permissions.IsAuthenticated,)
    authentication_classes = (JwtAuthentication, SessionAuthentication,)
    throttle_classes = (ServiceUserThrottle,)

    REQUIRED_PARAM_EMAIL = 'email'
    REQUIRED_PARAM_ENTERPRISE_NAME = 'enterprise_name'
    OPTIONAL_PARAM_NUMBER_OF_CODES = 'number_of_codes'
    OPTIONAL_PARAM_NOTES = 'notes'

    MISSING_REQUIRED_PARAMS_MSG = "Some required parameter(s) missing: {}"

    def get_required_query_params(self, request):
        """
        Gets ``email``, ``enterprise_name``, ``number_of_codes``, and ``notes``,
        which are the relevant parameters for this API endpoint.

        :param request: The request to this endpoint.
        :return: The ``email``, ``enterprise_name``, ``number_of_codes`` and ``notes`` from the request.
        """
        email = get_request_value(request, self.REQUIRED_PARAM_EMAIL, '')
        enterprise_name = get_request_value(request, self.REQUIRED_PARAM_ENTERPRISE_NAME, '')
        number_of_codes = get_request_value(request, self.OPTIONAL_PARAM_NUMBER_OF_CODES, '')
        notes = get_request_value(request, self.OPTIONAL_PARAM_NOTES, '')
        if not (email and enterprise_name):
            raise CodesAPIRequestError(
                self.get_missing_params_message([
                    (self.REQUIRED_PARAM_EMAIL, bool(email)),
                    (self.REQUIRED_PARAM_ENTERPRISE_NAME, bool(enterprise_name)),
                ])
            )
        return email, enterprise_name, number_of_codes, notes

    def get_missing_params_message(self, parameter_state):
        """
        Get a user-friendly message indicating a missing parameter for the API endpoint.
        """
        params = ', '.join(name for name, present in parameter_state if not present)
        return self.MISSING_REQUIRED_PARAMS_MSG.format(params)

    @permission_required('enterprise.can_access_admin_dashboard')
    def post(self, request):
        """
        POST /enterprise/api/v1/request_codes

        Requires a JSON object of the following format::

            {
                "email": "bob@alice.com",
                "enterprise_name": "IBM",
                "number_of_codes": "50",
                "notes": "Help notes for codes request",
            }

        Keys:
            email: Email of the customer who has requested more codes.
            enterprise_name: The name of the enterprise requesting more codes.
            number_of_codes: The number of codes requested.
            notes: Help notes related to codes request.
        """
        try:
            email, enterprise_name, number_of_codes, notes = self.get_required_query_params(request)
        except CodesAPIRequestError as invalid_request:
            return Response({'error': str(invalid_request)}, status=HTTP_400_BAD_REQUEST)

        subject_line = _('Code Management - Request for Codes by {token_enterprise_name}').format(
            token_enterprise_name=enterprise_name
        )
        body_msg = create_message_body(email, enterprise_name, number_of_codes, notes)
        app_config = apps.get_app_config("enterprise")
        from_email_address = app_config.enterprise_integrations_email
        cs_email = app_config.customer_success_email
        data = {
            self.REQUIRED_PARAM_EMAIL: email,
            self.REQUIRED_PARAM_ENTERPRISE_NAME: enterprise_name,
            self.OPTIONAL_PARAM_NUMBER_OF_CODES: number_of_codes,
            self.OPTIONAL_PARAM_NOTES: notes,
        }
        try:
            messages_sent = mail.send_mail(
                subject_line,
                body_msg,
                from_email_address,
                [cs_email],
                fail_silently=False
            )
            LOGGER.info('[Enterprise API] Coupon code request emails sent: %s', messages_sent)
            return Response(data, status=HTTP_200_OK)
        except SMTPException:
            error_message = _(
                '[Enterprise API] Failure in sending e-mail to support.'
                ' SupportEmail: {token_cs_email}, UserEmail: {token_email}, EnterpriseName: {token_enterprise_name}'
            ).format(
                token_cs_email=cs_email,
                token_email=email,
                token_enterprise_name=enterprise_name
            )
            LOGGER.error(error_message)
            return Response(
                {'error': 'Request codes email could not be sent'},
                status=HTTP_500_INTERNAL_SERVER_ERROR
            )



# Generated by Django 3.2.16 on 2022-11-22 10:09

from django.db import migrations, models
import enterprise.validators


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0164_enterprisecatalogquery_include_exec_ed_2u_courses'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerreportingconfiguration',
            name='pgp_encryption_key',
            field=models.TextField(blank=True, help_text='The key for encryption, if PGP encrypted file is required.', null=True, validators=[enterprise.validators.validate_pgp_key], verbose_name='PGP Encryption Key'),
        ),
    ]



"""
Django management command to explore Exec Ed inclusion flag migration
"""

import copy
import json
import logging

from requests.exceptions import HTTPError

from django.core.management import BaseCommand

from enterprise.api_client.discovery import CourseCatalogApiServiceClient
from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
from enterprise.models import EnterpriseCatalogQuery, EnterpriseCustomerCatalog
from integrated_channels.utils import batch_by_pk

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    """
    Enumerate the catalog filters and log information about how we might migrate them.
    """

    def handle(self, *args, **options):
        enterprise_catalog_client = EnterpriseCatalogApiClient()
        discovery_client = CourseCatalogApiServiceClient()

        for catalog_query_batch in batch_by_pk(EnterpriseCatalogQuery):
            for catalog_query in catalog_query_batch:
                logger.info(f'{catalog_query.id} {catalog_query.include_exec_ed_2u_courses}')

                if catalog_query.include_exec_ed_2u_courses:
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs '
                        f'query {catalog_query.id} already includes exec ed'
                    )
                    continue

                if catalog_query.content_filter.get('course_type'):
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs '
                        f'query {catalog_query.id} already references course_type somehow'
                    )
                    continue

                if catalog_query.content_filter.get('aggregation_key'):
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs '
                        f'query {catalog_query.id} references aggregation_key somehow'
                    )
                    continue

                new_content_filter = copy.deepcopy(catalog_query.content_filter)
                new_content_filter['course_type__exclude'] = 'executive-education-2u'
                new_content_filter_json = json.dumps(new_content_filter)
                logger.info(
                    'compare_discovery_and_enterprise_catalogs '
                    f'query {catalog_query.id} new filter: {new_content_filter_json}'
                )

        for cusrtomer_catalog_batch in batch_by_pk(EnterpriseCustomerCatalog):
            for customer_catalog in cusrtomer_catalog_batch:
                logger.info(f'{customer_catalog.uuid}')

                if customer_catalog.content_filter.get('course_type'):
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs '
                        f'catalog {customer_catalog.uuid} already references course_type somehow'
                    )
                    continue

                if customer_catalog.content_filter.get('aggregation_key'):
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs '
                        f'catalog {customer_catalog.uuid} references aggregation_key somehow'
                    )
                    continue

                try:
                    old_content_filter = customer_catalog.content_filter
                    new_content_filter = copy.deepcopy(customer_catalog.content_filter)
                    new_content_filter['course_type__exclude'] = 'executive-education-2u'
                    new_content_filter_json = json.dumps(new_content_filter)
                    old_discovery_count = discovery_client.get_catalog_results_from_discovery(old_content_filter).get('count')  # pylint: disable=line-too-long
                    new_discovery_count = discovery_client.get_catalog_results_from_discovery(new_content_filter).get('count')  # pylint: disable=line-too-long
                    enterprise_count = enterprise_catalog_client.get_catalog_content_count(customer_catalog.uuid)
                    logger.info(
                        'compare_discovery_and_enterprise_catalogs catalog '
                        f'{customer_catalog.uuid} '
                        f'existing discovery count: {old_discovery_count}, '
                        f'new discovery count: {new_discovery_count}, '
                        f'existing enterprise count: {enterprise_count}, '
                        f'new filter: {new_content_filter_json}'
                    )
                except HTTPError:
                    logger.exception(
                        'compare_discovery_and_enterprise_catalogs '
                        f'error checking catalog {customer_catalog.uuid}'
                    )



# Generated by Django 3.2.23 on 2024-01-17 12:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0030_merge_0028_auto_20231116_1826_0029_auto_20231106_1233'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='password',
        ),
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='token',
        ),
        migrations.RemoveField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='username',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='password',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='token',
        ),
        migrations.RemoveField(
            model_name='moodleenterprisecustomerconfiguration',
            name='username',
        ),
    ]



"""
Assign skills to degreed courses
"""
from logging import getLogger

from requests.exceptions import ConnectionError, RequestException, Timeout  # pylint: disable=redefined-builtin

from django.contrib import auth
from django.core.management.base import BaseCommand, CommandError

from enterprise.api_client.enterprise_catalog import EnterpriseCatalogApiClient
from integrated_channels.degreed2.client import Degreed2APIClient
from integrated_channels.exceptions import ClientError
from integrated_channels.integrated_channel.management.commands import IntegratedChannelCommandMixin
from integrated_channels.utils import generate_formatted_log

User = auth.get_user_model()
LOGGER = getLogger(__name__)


class Command(IntegratedChannelCommandMixin, BaseCommand):
    """
    Add skill metadata to existing Degreed courses.

    ./manage.py lms assign_skills_to_degreed_courses
    """

    def add_arguments(self, parser):
        """
        Add required arguments to the parser.
        """
        parser.add_argument(
            '--catalog_user',
            dest='catalog_user',
            required=True,
            metavar='ENTERPRISE_CATALOG_API_USERNAME',
            help='Use this user to access the Enterprise Catalog API.'
        )
        super().add_arguments(parser)

    def _prepare_json_payload_for_skills_endpoint(self, course_skills):
        """
        Prepares a json payload for skills in the Degreed expected format.
        """
        course_skills_json = []
        for skill in course_skills:
            skill_data = {"type": "skills", "id": skill}
            course_skills_json.append(skill_data)
        return {
            "data": course_skills_json
        }

    def handle(self, *args, **options):
        """
        Update all existing Degreed courses to assign skills metadata.
        """
        options['channel'] = 'DEGREED2'
        username = options['catalog_user']

        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist as no_user_error:
            raise CommandError('A user with the username {} was not found.'.format(username)) from no_user_error

        enterprise_catalog_client = EnterpriseCatalogApiClient(user)
        integrated_channels = self.get_integrated_channels(options)
        for degreed_channel_config in integrated_channels:
            enterprise_customer = degreed_channel_config.enterprise_customer
            enterprise_customer_catalogs = degreed_channel_config.customer_catalogs_to_transmit or \
                enterprise_customer.enterprise_customer_catalogs.all()
            try:
                content_metadata_in_catalogs = enterprise_catalog_client.get_content_metadata(
                    enterprise_customer,
                    enterprise_customer_catalogs
                )
            except (RequestException, ConnectionError, Timeout) as exc:
                LOGGER.exception(
                    'Failed to retrieve enterprise catalogs content metadata due to: [%s]', str(exc)
                )
                continue

            degreed_client = Degreed2APIClient(degreed_channel_config)
            LOGGER.info(
                generate_formatted_log(
                    degreed_channel_config.channel_code(),
                    enterprise_customer.uuid,
                    None,
                    None,
                    f'[Degreed Skills] Attempting to assign skills for customer {enterprise_customer.slug}'
                )
            )

            for content_item in content_metadata_in_catalogs:
                course_id = content_item.get('key', None)
                course_skills = content_item.get('skill_names', [])

                # if we get empty list of skills, there's no point making API call to Degreed.
                if not course_skills:
                    continue

                json_payload = self._prepare_json_payload_for_skills_endpoint(course_skills)

                # assign skills metadata to degreed course by first fetching degreed course id
                try:
                    degreed_client.assign_course_skills(course_id, json_payload)
                except ClientError as error:
                    LOGGER.error(
                        generate_formatted_log(
                            degreed_channel_config.channel_code(),
                            enterprise_customer.uuid,
                            None,
                            None,
                            f'Degreed2APIClient assign_course_skills failed for course {course_id} '
                            f'with message: {error.message}'
                        )
                    )
                    continue
                except RequestException as error:
                    LOGGER.error(
                        generate_formatted_log(
                            degreed_channel_config.channel_code(),
                            enterprise_customer.uuid,
                            None,
                            None,
                            f'Degreed2APIClient request to assign skills failed with message: {error.message}'
                        )
                    )
                    continue



# Generated by Django 2.2.19 on 2022-05-12 14:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0155_add_is_relinkable_to_enterprise_customer_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalsystemwideenterpriseuserroleassignment',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='systemwideenterpriseuserroleassignment',
            name='is_active',
            field=models.BooleanField(default=True),
        ),
    ]




"""
Views for the ``enterprise-user`` API endpoint.
"""

from collections import OrderedDict

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters, permissions, response, status
from rest_framework.pagination import PageNumberPagination

from django.contrib import auth
from django.core.exceptions import ValidationError
from django.db.models import Q

from enterprise import models
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadOnlyModelViewSet
from enterprise.logging import getEnterpriseLogger

User = auth.get_user_model()

LOGGER = getEnterpriseLogger(__name__)


class EnterpriseCustomerSupportPaginator(PageNumberPagination):
    """Custom paginator for the enterprise customer support."""
    page_size = 8

    def get_paginated_response(self, data):
        """Return a paginated style `Response` object for the given output data."""
        return response.Response(
            OrderedDict([
                ('count', self.page.paginator.count),
                ('num_pages', self.page.paginator.num_pages),
                ('next', self.get_next_link()),
                ('previous', self.get_previous_link()),
                ('results', data),
            ])
        )

    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a page object,
        or `None` if pagination is not configured for this view.

        """
        if isinstance(queryset, filter):
            queryset = list(queryset)

        return super().paginate_queryset(queryset, request, view)


class EnterpriseCustomerSupportViewSet(EnterpriseReadOnlyModelViewSet):
    """
    API views for the ``enterprise-customer-support`` API endpoint.
    """
    queryset = models.PendingEnterpriseCustomerUser.objects.all()
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter)
    permission_classes = (permissions.IsAuthenticated,)
    paginator = EnterpriseCustomerSupportPaginator()

    ordering_fields = ['id']
    filterset_fields = ['user_email']

    def filter_queryset_by_email(self, queryset, is_pending_user=False):
        """
        Filter queryset based on user provided email address
        """
        filter_email = self.request.query_params.get('user_email', None)

        if filter_email:
            if not is_pending_user:
                queryset = queryset.filter(
                    user_id__in=User.objects.filter(Q(email__icontains=filter_email))
                )
            else:
                queryset = queryset.filter(user_email=filter_email)

        return queryset

    def retrieve(self, request, *args, **kwargs):
        """
        Filter down the queryset of groups available to the requesting uuid.
        """
        enterprise_uuid = kwargs.get('enterprise_uuid', None)
        users = []

        try:
            enterprise_customer_queryset = models.EnterpriseCustomerUser.objects.filter(
                enterprise_customer__uuid=enterprise_uuid,
            )
            enterprise_customer_queryset = self.filter_queryset_by_email(enterprise_customer_queryset)
            users.extend(enterprise_customer_queryset)

            pending_enterprise_customer_queryset = models.PendingEnterpriseCustomerUser.objects.filter(
                enterprise_customer__uuid=enterprise_uuid
            ).order_by('user_email')
            pending_enterprise_customer_queryset = self.filter_queryset_by_email(
                pending_enterprise_customer_queryset,
                is_pending_user=True
            )
            users.extend(pending_enterprise_customer_queryset)

        except ValidationError:
            # did not find UUID match in either EnterpriseCustomerUser or PendingEnterpriseCustomerUser
            return response.Response(
                {'detail': 'Could not find enterprise uuid {}'.format(enterprise_uuid)},
                status=status.HTTP_404_NOT_FOUND
            )

        # default sort criteria
        is_reversed = False
        sort_field = 'first_name'

        ordering_criteria = self.request.query_params.get('ordering', None)

        # apply pre-serialization ordering by user criteria before the users
        # get divvied up by pagination
        if ordering_criteria:
            is_reversed = '-' in ordering_criteria
            sort_field = 'user_id'

        # sort the users by default or specified criteria since the queryset will get
        # split up during pagination and the post-serialization sort operations
        # will be only applied to a single page of results
        users = sorted(
            users,
            key=(
                lambda k:
                getattr(k, sort_field)
                if hasattr(k, sort_field)
                else k.id
            ),
            reverse=is_reversed
        )

        # paginate the queryset
        users_page = self.paginator.paginate_queryset(
            users,
            request,
            view=self
        )

        # serialize the paged dataset
        serializer = serializers.EnterpriseUserSerializer(
            users_page,
            many=True
        )
        serializer_data = serializer.data

        # Apply post-serialization default ordering criteria (first by is_admin,
        # then first name) only if user does not specify ordering criteria;
        # Process this after the data has been serialized since the is_admin
        # field is computed/available only after serialization step
        if not ordering_criteria:
            serializer_data = sorted(
                serializer_data,
                key=lambda k: (
                    # sort by is_admin = True first (i.e. -1),
                    # then sort by first_name lexicographically
                    (-1 * k['is_admin'], k['enterprise_customer_user']['first_name'])
                    if k['enterprise_customer_user'] is not None
                    else -1 * k['is_admin']
                )
            )

        return self.paginator.get_paginated_response(serializer_data)
